load("Documents/My Teaching/Time Series/R Data/tsa3.rda")

Example 5.1
library(fracdiff)
lvarve = log(varve) - mean(log(varve))
varve.fd = fracdiff(lvarve, nar=0, nma=0, M=30)
varve.fd$d  # = 0.3841688
varve.fd$stderror.dpq  # = 4.589514e-06 (If you believe this, I have a bridge for sale.)
p = rep(1,31)
for (k in 1:30){ p[k+1] = (k-varve.fd$d)*p[k]/(k+1) }
plot(1:30, p[-1], ylab=expression(pi(d)), xlab="Index", type="h", lwd=2)
res.fd = diffseries(log(varve), varve.fd$d)       # frac diff resids            
res.arima = resid(arima(log(varve), order=c(1,1,1))) # arima resids
dev.new()
par(mfrow=c(2,1))  
acf(res.arima, 100, xlim=c(4,97), ylim=c(-.2,.2), main="arima resids")
acf(res.fd, 100, xlim=c(4,97), ylim=c(-.2,.2), main="frac diff resids")

Example 5.2
series = log(varve)  # specify series to be analyzed
d0 = .1              # initial value of d
n.per = nextn(length(series))
m = (n.per)/2  - 1
per = abs(fft(series-mean(series))[-1])^2  # remove 0 freq
per = per/n.per      # R doesn't scale fft by sqrt(n)
g = 4*(sin(pi*((1:m)/n.per))^2)
# Function to calculate -log.likelihood
whit.like = function(d){  
 g.d=g^d
 sig2 = (sum(g.d*per[1:m])/m)
 log.like = m*log(sig2) - d*sum(log(g)) + m
 return(log.like)   
 }
# Estimation (?optim for details - output not shown)
(est = optim(d0, whit.like, gr=NULL, method="L-BFGS-B", hessian=TRUE, lower=-.5, upper=.5, 
             control=list(trace=1,REPORT=1)))
# Results  [d.hat = .380, se(dhat) = .028]  
cat("d.hat =", est$par, "se(dhat) = ",1/sqrt(est$hessian),"\n")  
g.dhat = g^est$par
sig2 = sum(g.dhat*per[1:m])/m
cat("sig2hat =",sig2,"\n")  # sig2hat = .229 

u = spec.ar(log(varve), plot=FALSE)  # produces AR(8)       
g = 4*(sin(pi*((1:500)/2000))^2)
fhat = sig2*g^{-est$par}  # long memory spectral estimate          
plot(1:500/2000, log(fhat), type="l", ylab="log(spectrum)", xlab="frequency")
lines(u$freq[1:250], log(u$spec[1:250]), lty="dashed") 
ar.mle(log(varve))  # to get AR(8) estimates 

Example 5.3
library(tseries)
adf.test(log(varve), k=0)  # DF test
adf.test(log(varve))       # ADF test
pp.test(log(varve))        # PP test

Example 5.4
gnpgr = diff(log(gnp))  # get the returns
sarima(gnpgr, 1, 0, 0)  # fit an AR(1)
acf2(innov^2, 24)       # get (p)acf of the squared residuals
 
library(fGarch)
summary(garchFit(~arma(1,0)+garch(1,0), gnpgr))

Example 5.5
library(fGarch)
summary(nyse.g <- garchFit(~garch(1,1), nyse))
u = nyse.g@sigma.t
plot(window(nyse, start=900, end=1000), ylim=c(-.22,.2), ylab="NYSE Returns")
lines(window(nyse-2*u, start=900, end=1000), lty=2, col=4)
lines(window(nyse+2*u, start=900, end=1000), lty=2, col=4)

Example 5.6
plot(flu, type="c")  # first 3 lines plot the data with months as symbols
Months = c("J","F","M","A","M","J","J","A","S","O","N","D")
points(flu, pch=Months, cex=.8, font=2) 

dflu = diff(flu)
thrsh = .05  # threshold
Z = ts.intersect(dflu, lag(dflu,-1), lag(dflu,-2), lag(dflu,-3), lag(dflu,-4))
ind1 = ifelse(Z[,2] < thrsh, 1, NA)  # indicator < thrsh
ind2 = ifelse(Z[,2] < thrsh, NA, 1)  # indicator ≥ thrsh
X1 = Z[,1]*ind1
X2 = Z[,1]*ind2
summary(fit1<-lm(X1~Z[,2:5]))  # case 1
summary(fit2<-lm(X2~Z[,2:5]))  # case 2
D = cbind(rep(1, nrow(Z)), Z[,2:5])  # get predictions
b1 = fit1$coef 
b2 = fit2$coef
p1 = D%*%b1
p2=D%*%b2
prd = ifelse(Z[,2] < thrsh, p1, p2)
plot(dflu, type="p", pch=2, ylim=c(-.5,.5))
lines(prd, lty=2, col=2)
prde1 = sqrt(sum(resid(fit1)^2)/df.residual(fit1))
prde2 = sqrt(sum(resid(fit2)^2)/df.residual(fit2))
prde = ifelse(Z[,2] < thrsh, prde1, prde2)
lines(prd + 2*prde, col=3)
lines(prd - 2*prde, col=3)

Example 5.7
trend = time(cmort)
temp = tempr - mean(tempr)
temp2 = temp^2
fit = lm(cmort~trend + temp + temp2 + part, na.action=NULL)
acf2(resid(fit), 52)   # implies AR2
(fit2 = arima(cmort, order=c(2,0,0), xreg=cbind(trend,temp,temp2,part)))
dev.new()
acf2(resid(fit2), 52)  # no obvious departures from whiteness
(Q = Box.test(resid(fit2), 12, type="Ljung")$statistic)  # X-squared = 6.91                           
pchisq(as.numeric(Q), df=(12-2), lower=FALSE)  # p-value = .73

Example 5.8
acf2(soi)
(fit = arima(soi, xreg=time(soi), order=c(1, 0, 0)))       
ar1 = as.numeric(fit$coef[1])    # = 0.5875387
soi.pw = resid(fit)  
rec.d = resid(lm(rec~time(rec), na.action=NULL))  
rec.fil = filter(rec.d, filter=c(1, -ar1), method="conv", sides=1)
ccf(soi.pw, rec.fil, main="", ylab="CCF", na.action=na.omit)

Example 5.9
rec.d = resid(lm(rec~time(rec), na.action=NULL))  
soi.d = resid(lm(soi~time(soi), na.action=NULL))
fish = ts.intersect(rec.d, rec.d1=lag(rec.d,-1), soi.d5=lag(soi,-5), dframe=TRUE)
summary(fish.fit <- lm(rec.d~0+rec.d1+soi.d5, data=fish))
om1 = as.numeric(fish.fit$coef[1]) 
eta.hat = filter(resid(fish.fit), filter=c(1,-om1), method="recur", sides=1)
acf2(eta.hat)
(eta.fit <- arima(eta.hat, order=c(3,0,0)))

Example 5.10
library(vars)
x = cbind(cmort, tempr, part)
summary(VAR(x, p=1, type="both"))  # "both" fits constant + trend

Example 5.11
# continued from 5.10
VARselect(x, lag.max=10, type="both")
summary(fit <- VAR(x, p=2, type="both"))
acf(resid(fit), 52)
serial.test(fit, lags.pt=12, type="PT.adjusted")
(fit.pr = predict(fit, n.ahead = 24, ci = 0.95))  # 4 weeks ahead
dev.new()
fanchart(fit.pr)  # plot prediction + error