#############################################################
#     This is ssall.R, where you get it all and more!
#                  version 7.21.2009
# 
#        You get: Kfilter0, Ksmooth0  (level 0 - see below)
#                 Kfilter1, Ksmooth1  (level 1 - see below)
#                 Kfilter2, Ksmooth2  (level 2 - see below)
#                     
#############################################################
#  LEVEL 0: state space modeling (easy stuff, no inputs)
#           the filter and smoother with fixed measurement 
#           matrix A(t)= A for all t
#
#  LEVEL 1: state space modeling 
#           (i) allows inputs, u(t) 
#           (ii) the measurement matrix is a sequence A(t)
#           --read the notes in the filter for special cases--
#
#      LEVELS 0 and 1 use the Oarray package (included here)
#
#  LEVEL 2: state space modeling  
#          (i) allows inputs, u(t) 
#          (ii) the measurement matrix is a sequence A(t)
#          (iii) errors can be correlated
#
# !!!! check out the notes for each function below for particulars  !!!!!
#
#############################################################
#############################################################
#********************* start LEVEL 0 ***********************
#############################################################
#   the filter
#############################################################
Kfilter0 = function(num,y,A,mu0,Sigma0,Phi,cQ,cR){
  #
  # NOTE: must give cholesky decomp: cQ=chol(Q), cR=chol(R)
 Q=t(cQ)%*%cQ
 R=t(cR)%*%cR
   # y is num by q  (time=row series=col)
   # A is a q by p matrix
   # R is q by q
   # mu0 is p by 1
   # Sigma0, Phi, Q are p by p
 Phi=as.matrix(Phi)
 pdim=nrow(Phi)    
 y=as.matrix(y)
 qdim=ncol(y)
 xp=array(NA, dim=c(pdim,1,num))         # xp=x_t^{t-1}          
 Pp=array(NA, dim=c(pdim,pdim,num))      # Pp=P_t^{t-1}
 xf=array(NA, dim=c(pdim,1,num))         # xf=x_t^t
 Pf=array(NA, dim=c(pdim,pdim,num))      # Pf=x_t^t
 innov=array(NA, dim=c(qdim,1,num))      # innovations
 sig=array(NA, dim=c(qdim,qdim,num))     # innov var-cov matrix
# initialize (because R can't count from zero)
   x00=as.matrix(mu0, nrow=pdim, ncol=1)
   P00=as.matrix(Sigma0, nrow=pdim, ncol=pdim)
   xp[,,1]=Phi%*%x00
   Pp[,,1]=Phi%*%P00%*%t(Phi)+Q
     sigtemp=A%*%Pp[,,1]%*%t(A)+R
   sig[,,1]=(t(sigtemp)+sigtemp)/2     # innov var - make sure it's symmetric
   siginv=solve(sig[,,1])          
   K=Pp[,,1]%*%t(A)%*%siginv
   innov[,,1]=y[1,]-A%*%xp[,,1]
   xf[,,1]=xp[,,1]+K%*%innov[,,1]
   Pf[,,1]=Pp[,,1]-K%*%A%*%Pp[,,1]
   sigmat=as.matrix(sig[,,1], nrow=qdim, ncol=qdim)
   like = log(det(sigmat)) + t(innov[,,1])%*%siginv%*%innov[,,1]   # -log(likelihood)
########## start filter iterations ###################
 for (i in 2:num){
  xp[,,i]=Phi%*%xf[,,i-1]
  Pp[,,i]=Phi%*%Pf[,,i-1]%*%t(Phi)+Q
     sigtemp=A%*%Pp[,,i]%*%t(A)+R
   sig[,,i]=(t(sigtemp)+sigtemp)/2     # innov var - make sure it's symmetric
   siginv=solve(sig[,,i])              
  K=Pp[,,i]%*%t(A)%*%siginv
  innov[,,i]=y[i,]-A%*%xp[,,i]
  xf[,,i]=xp[,,i]+K%*%innov[,,i]
  Pf[,,i]=Pp[,,i]-K%*%A%*%Pp[,,i]
    sigmat=as.matrix(sig[,,i], nrow=qdim, ncol=qdim)
  like= like + log(det(sigmat)) + t(innov[,,i])%*%siginv%*%innov[,,i]
  }
    like=0.5*like
    list(xp=xp,Pp=Pp,xf=xf,Pf=Pf,like=like,innov=innov,sig=sig,Kn=K)
}
##########################################################
# end filter 
##########################################################
# the smoother
##########################################################
Ksmooth0 = function(num,y,A,mu0,Sigma0,Phi,cQ,cR){
#
# Note: Q and R are given as Cholesky decomps
#       cQ=chol(Q), cR=chol(R)
#
 kf=Kfilter0(num,y,A,mu0,Sigma0,Phi,cQ,cR)
 pdim=nrow(as.matrix(Phi))  
 xs=array(NA, dim=c(pdim,1,num))      # xs=x_t^n
 Ps=array(NA, dim=c(pdim,pdim,num))   # Ps=P_t^n
 J=array(NA, dim=c(pdim,pdim,num))    # J=J_t
 xs[,,num]=kf$xf[,,num] 
 Ps[,,num]=kf$Pf[,,num]
 for(k in num:2)  {
 J[,,k-1]=(kf$Pf[,,k-1]%*%t(Phi))%*%solve(kf$Pp[,,k])
 xs[,,k-1]=kf$xf[,,k-1]+J[,,k-1]%*%(xs[,,k]-kf$xp[,,k])
 Ps[,,k-1]=kf$Pf[,,k-1]+J[,,k-1]%*%(Ps[,,k]-kf$Pp[,,k])%*%t(J[,,k-1])
}
# and now for the initial values because R can't count backward to zero
    x00=mu0
    P00=Sigma0
   J0=as.matrix((P00%*%t(Phi))%*%solve(kf$Pp[,,1]), nrow=pdim, ncol=pdim)
   x0n=as.matrix(x00+J0%*%(xs[,,1]-kf$xp[,,1]), nrow=pdim, ncol=1)
   P0n= P00 + J0%*%(Ps[,,k]-kf$Pp[,,k])%*%t(J0)
list(xs=xs,Ps=Ps,x0n=x0n,P0n=P0n,J0=J0,J=J,xp=kf$xp,Pp=kf$Pp,xf=kf$xf,Pf=kf$Pf,like=kf$like,Kn=kf$K)
} 
############# end smooth ####################################
################## end level 0 ##############################
#************************************************************
#******************* start LEVEL 1 **************************
#############################################################
#   the filter
#############################################################
Kfilter1 = function(num,y,A,mu0,Sigma0,Phi,Ups,Gam,cQ,cR,input){
  #
  # NOTE: must give cholesky decomp: cQ=chol(Q), cR=chol(R)
 Q=t(cQ)%*%cQ
 R=t(cR)%*%cR
   # y is num by q  (time=row series=col)
   # input is num by r (use 0 if not needed)
   # A is an array with dim=c(q,p,num)
   # Ups is p by r  (use 0 if not needed)
   # Gam is q by r  (use 0 if not needed)
   # R is q by q
   # mu0 is p by 1
   # Sigma0, Phi, Q are p by p
 Phi=as.matrix(Phi)
 pdim=nrow(Phi)     
 y=as.matrix(y)
 qdim=ncol(y)
 rdim=ncol(as.matrix(input))
  if (Ups==0) Ups = matrix(0,pdim,rdim)
  if (Gam==0) Gam = matrix(0,qdim,rdim)
 ut=matrix(input,num,rdim)
 xp=array(NA, dim=c(pdim,1,num))         # xp=x_t^{t-1}          
 Pp=array(NA, dim=c(pdim,pdim,num))      # Pp=P_t^{t-1}
 xf=array(NA, dim=c(pdim,1,num))         # xf=x_t^t
 Pf=array(NA, dim=c(pdim,pdim,num))      # Pf=x_t^t
 innov=array(NA, dim=c(qdim,1,num))      # innovations
 sig=array(NA, dim=c(qdim,qdim,num))     # innov var-cov matrix
# initialize (because R can't count from zero) 
 x00=as.matrix(mu0, nrow=pdim, ncol=1)
   P00=as.matrix(Sigma0, nrow=pdim, ncol=pdim)
   xp[,,1]=Phi%*%x00 + Ups%*%ut[1,]
   Pp[,,1]=Phi%*%P00%*%t(Phi)+Q
     B = matrix(A[,,1], nrow=qdim, ncol=pdim)  
     sigtemp=B%*%Pp[,,1]%*%t(B)+R
   sig[,,1]=(t(sigtemp)+sigtemp)/2     # innov var - make sure it's symmetric
   siginv=solve(sig[,,1])          
   K=Pp[,,1]%*%t(B)%*%siginv
   innov[,,1]=y[1,]-B%*%xp[,,1]-Gam%*%ut[1,]
   xf[,,1]=xp[,,1]+K%*%innov[,,1]
   Pf[,,1]=Pp[,,1]-K%*%B%*%Pp[,,1]
   sigmat=as.matrix(sig[,,1], nrow=qdim, ncol=qdim)
   like = log(det(sigmat)) + t(innov[,,1])%*%siginv%*%innov[,,1]   # -log(likelihood)
############################# 
# start filter iterations
#############################
 for (i in 2:num){
  xp[,,i]=Phi%*%xf[,,i-1] + Ups%*%ut[i,]
  Pp[,,i]=Phi%*%Pf[,,i-1]%*%t(Phi)+Q
      B = matrix(A[,,i], nrow=qdim, ncol=pdim)  
      siginv=B%*%Pp[,,i]%*%t(B)+R
  sig[,,i]=(t(siginv)+siginv)/2     # make sure sig is symmetric
    siginv=solve(sig[,,i])          # now siginv is sig[[i]]^{-1}
  K=Pp[,,i]%*%t(B)%*%siginv
  innov[,,i]=y[i,]-B%*%xp[,,i]-Gam%*%ut[i,]
  xf[,,i]=xp[,,i]+K%*%innov[,,i]
  Pf[,,i]=Pp[,,i]-K%*%B%*%Pp[,,i]
    sigmat=matrix(sig[,,i], nrow=qdim, ncol=qdim)
  like= like + log(det(sigmat)) + t(innov[,,i])%*%siginv%*%innov[,,i]
  }
    like=0.5*like
    list(xp=xp,Pp=Pp,xf=xf,Pf=Pf,like=like,innov=innov,sig=sig,Kn=K)
}
##########################################################
#   end filter
##########################################################
#  the smoother
###########################################################
Ksmooth1 = function(num,y,A,mu0,Sigma0,Phi,Ups,Gam,cQ,cR,input){
#
# Note: Q and R are given as Cholesky decomps
#       cQ=chol(Q), cR=chol(R)
#  Use Ups=0 or Gam=0 or input=0 if these aren't needed
#
 kf=Kfilter1(num,y,A,mu0,Sigma0,Phi,Ups,Gam,cQ,cR,input)
 pdim=nrow(as.matrix(Phi))  
 xs=array(NA, dim=c(pdim,1,num))      # xs=x_t^n
 Ps=array(NA, dim=c(pdim,pdim,num))   # Ps=P_t^n
 J=array(NA, dim=c(pdim,pdim,num))    # J=J_t
 xs[,,num]=kf$xf[,,num] 
 Ps[,,num]=kf$Pf[,,num]
 for(k in num:2)  {
 J[,,k-1]=(kf$Pf[,,k-1]%*%t(Phi))%*%solve(kf$Pp[,,k])
 xs[,,k-1]=kf$xf[,,k-1]+J[,,k-1]%*%(xs[,,k]-kf$xp[,,k])
 Ps[,,k-1]=kf$Pf[,,k-1]+J[,,k-1]%*%(Ps[,,k]-kf$Pp[,,k])%*%t(J[,,k-1])
}
# and now for the initial values because R can't count backward to zero
    x00=mu0
    P00=Sigma0
   J0=as.matrix((P00%*%t(Phi))%*%solve(kf$Pp[,,1]), nrow=pdim, ncol=pdim)
   x0n=as.matrix(x00+J0%*%(xs[,,1]-kf$xp[,,1]), nrow=pdim, ncol=1)
   P0n= P00 + J0%*%(Ps[,,k]-kf$Pp[,,k])%*%t(J0)
list(xs=xs,Ps=Ps,x0n=x0n,P0n=P0n,J0=J0,J=J,xp=kf$xp,Pp=kf$Pp,xf=kf$xf,Pf=kf$Pf,like=kf$like,Kn=kf$K)
} 
################## end level 1 ###################################
#*****************************************************************
#******************** start LEVEL 2 ******************************
##################################################################
#   the filter
##################################################################
Kfilter2 = function(num,y,A,mu1,Sigma1,Phi,Ups,Gam,cQ,cR,S,input){
  #
  ######## Reference Property 6.5 in Section 6.6 ###########
  # num is the number of observations
  # y is the data matrix (num by q)
  # A has to be an array with dim=c(q,p,num)
  # mu1= E(x_1) = x_1^0 = Phi%*%mu0 + Gam%*%input0 
  # Sigma1 = var(x_1)= P_1^0 =  Phi%*%Sigma0%*%t(Phi)+Q
  # input has to be a matrix (num by r) - similar to obs y
  # set Ups or Gam or input to 0 if not used
  # Must give Cholesky decomp: cQ=chol(Q), cR=chol(R)  
Q=t(cQ)%*%cQ
R=t(cR)%*%cR 
  #
 Phi=as.matrix(Phi)
 pdim=nrow(Phi) 
 y=as.matrix(y)
 qdim=ncol(y)
 rdim=ncol(as.matrix(input))
  if (Ups==0) Ups = matrix(0,pdim,rdim)
  if (Gam==0) Gam = matrix(0,qdim,rdim)
 ut=matrix(input,num,rdim)
 xp=array(NA, dim=c(pdim,1,num+1))      # xp=x_t^{t-1}          
 Pp=array(NA, dim=c(pdim,pdim,num+1))   # Pp=P_t^{t-1}
 xf=array(NA, dim=c(pdim,1,num+1))      # xf=x_t^{t} 
 Pf=array(NA, dim=c(pdim,pdim,num+1))   # Pf=P_t^{t}
 Gain=array(NA, dim=c(pdim,qdim,num))
 innov=array(NA, dim=c(qdim,1,num))   # innovations
 sig=array(NA, dim=c(qdim,qdim,num))  # innov var-cov matrix
 like=0                               # -log(likelihood)
 xp[,,1]=mu1
 Pp[,,1]=Sigma1
 for (i in 1:num){
	B = matrix(A[,,i], nrow=qdim, ncol=pdim) 
   innov[,,i] = y[i,]-B%*%xp[,,i]-Gam%*%ut[i,]
    sigma = B%*%Pp[,,i]%*%t(B)+R 
    sigma=(t(sigma)+sigma)/2     # make sure sig is symmetric
   sig[,,i]=sigma
    siginv=solve(sigma)
   Gain[,,i]=(Phi%*%Pp[,,i]%*%t(B)+S)%*%siginv 
    K=Gain[,,i]
   xp[,,i+1]=Phi%*%xp[,,i] + Ups%*%ut[i,] + K%*%innov[,,i]
   Pp[,,i+1]=Phi%*%Pp[,,i]%*%t(Phi)+ Q - K%*%sig[,,i]%*%t(K)
   xf[,,i]=xp[,,i]+ Pp[,,i]%*%t(B)%*%siginv%*%innov[,,i]
   Pf[,,i]=Pp[,,i] - Pp[,,i]%*%t(B)%*%siginv%*%B%*%Pp[,,i] 
     sigma=matrix(sigma, nrow=qdim, ncol=qdim)
   like= like + log(det(sigma)) + t(innov[,,i])%*%siginv%*%innov[,,i]
  }	 
  like=0.5*like
  list(xp=xp,Pp=Pp,xf=xf,Pf=Pf, K=Gain,like=like,innov=innov,sig=sig)
}
#
##########################################################
#  the smoother
###########################################################
Ksmooth2 = function(num,y,A,mu1,Sigma1,Phi,Ups,Gam,cQ,cR,S,input){
#
# Note: Q and R are given as Cholesky decomps
#       cQ=chol(Q), cR=chol(R)
#  Use Ups=0 or Gam=0 or input=0 if these aren't needed
#
 kf=Kfilter2(num,y,A,mu1,Sigma1,Phi,Ups,Gam,cQ,cR,S,input)
 pdim=nrow(as.matrix(Phi))  
 xs=array(NA, dim=c(pdim,1,num))      # xs=x_t^n
 Ps=array(NA, dim=c(pdim,pdim,num))   # Ps=P_t^n
 J=array(NA, dim=c(pdim,pdim,num))    # J=J_t
 xs[,,num]=kf$xf[,,num]      
 Ps[,,num]=kf$Pf[,,num]
 for(k in num:2)  {
 J[,,k-1]=(kf$Pf[,,k-1]%*%t(Phi))%*%solve(kf$Pp[,,k])
 xs[,,k-1]=kf$xf[,,k-1]+J[,,k-1]%*%(xs[,,k]-kf$xp[,,k])
 Ps[,,k-1]=kf$Pf[,,k-1]+J[,,k-1]%*%(Ps[,,k]-kf$Pp[,,k])%*%t(J[,,k-1])
}
list(xs=xs,Ps=Ps,J=J,xp=kf$xp,Pp=kf$Pp,xf=kf$xf,Pf=kf$Pf,like=kf$like,Kn=kf$K)
} 
################## end ss2 #######################################
cat("  You now have", "\n")
cat("  Kfilter0, Ksmooth0", "\n")
cat("  Kfilter1, Ksmooth1", "\n")
cat("  Kfilter2, Ksmooth2", "\n")
# end ###################################