library(TSA)
library(foreign)
saber <- read.dta("Documents/My Research/Book Project 3/saber rattling dependent variables.dta")
attach(saber)
summary(saber)

# Saber Rattling Data
t <- length(anythreat_sum)
t
sr <- as.ts(anythreat_sum, start= c(1945,4), frequency=12)
ts.plot(sr ,xlab='month',ylab='Threats',main='Presidential Saber Rattling')

#Computing and Plotting the Periodogram and Spectrum Using TSA
# Raw Periodogram without connecting lines
sr.pgram <- periodogram(sr,ylab='Variable sr Periodogram',main='Raw Periodogram');  abline(h=0)
sr.pgram$spec

# Spikes at 3, 16, and 17. There are 384 estimated Fourier frequencies, with spikes at frequencies 3/766, 16/766, and 17/766
# This implies implies periodicities at 255 months, 47.88 months (four years), and 45 months. Smoothing diminishes the spikes
# at 255 and 45, leaving the spike at 48 months.
# This periodicity implies an election related phenomenon.

# Computing the Spectrum Using the base package
par(family="serif", font=1)
par(mfrow=c(3,1))
sp.unsmoothed <- spectrum(sr, log=c("no"), demean=TRUE, ylab="Power", xlab="Frequency", main = "Periodogram of Saber Rattling Time Series")
sp.unsmoothed$spec
sp.smoothed <- spectrum(sr, spans=3, log=c("no"), demean=TRUE, ylab="Power", xlab="Frequency", main="Smoothed Spectrum of Saber Rattling Time Series")
sp.smoothed$spec
sp.smoothed2 <- spectrum(sr, spans=6, log=c("no"), demean= TRUE, ylab="Power", xlab="Frequency", main="Figure 3.3: Spectral Decomposition of Saber Rattling Time Series")
sp.smoothed2$spec


sr.all <- as.ts(anythreat_sum, start= c(1945,4), frequency=12)
sr.gen <- as.ts(general, start= c(1945,4), frequency=12)
sr.specific <-as.ts(country, start= c(1945,4), frequency=12)

dev.new(width=8.5, height=10.5,pointsize=8)
par(family="serif", font=1, cex.main=2, cex.lab=1.5,cex.axis=1.5, cex.sub=1.25)
par(mfrow=c(3,1))
par(omi=c(1,1,1,1))                      #set the size of the outer margins
sp.all <- spectrum(sr.all, spans=5, log=c("no"), demean= TRUE, ylab="Power", xlab="Frequency", main="All Types Saber Rattling")
abline(v=16/766, lty="dotted")
sp.gen <- spectrum(sr.gen, spans=5, log=c("no"), demean= TRUE, ylab="Power", xlab="Frequency", main="General Saber Rattling")
abline(v=16/766, lty="dotted")
sp.country <- spectrum(sr.specific, spans=5, log=c("no"), demean= TRUE, ylab="Power", xlab="Frequency", main="Country-Specific")
abline(v=16/766, lty="dotted")
mtext("FIGURE 6: Spectra of Presidential Saber Rattling Time Series",3,outer=TRUE,line=1,cex=2.0, font=2)    #note that we seem to need to add the global title last
quartz.options(type="png")

# Using the correlogram to do the same thing.
# Spikes at 47 and 96 months
sr.acf <- acf(sr, lag.max=200)
sr.acf
